/*---------------------------------------------------------------------------*\
File
    readForceBalanceControls.H

Description
    Reads in the parameters for the force balance part of the solver.

*   The parameters which decide the basic operation of the module are
    only read at the beginning of a run, and cannot be modified during a run
    (defined in file: initializeForceBalance.H")

*   Parameters which define the various properties and constants are read
    at each time step and hence can be modified during run-time
\*---------------------------------------------------------------------------*/

    // The following parameters are read at each time step

    if(forceBalanceEnabled)
    {

       // Mass of moving system
       if(forceFoamDict.found("M"))
       {
          bodyMass = dimensionedScalar(forceFoamDict.lookup("M"));
          if(bodyMass.value() <= 0.0) bodyMass.value() = 1.0;
       }
       
       // Inertia of moving system
        if(forceFoamDict.found("inertia"))
        {
           bodyInertia = dimensionedScalar(forceFoamDict.lookup("inertia"));
           if(bodyInertia.value() <= 0.0) bodyInertia.value() = 1.0;
        }
       // Limit the maximum acceleration possible on the system (Bandwidth limitation)
       if(forceFoamDict.found("accelLim"))
       {
          accelLim = dimensionedScalar(forceFoamDict.lookup("accelLim"));
       }

       // Acceleration due to gravity
       if(forceFoamDict.found("g"))
       {
          gravityAccel = dimensionedScalar(forceFoamDict.lookup("g"));
          if(gravityAccel.value() < 0.0) gravityAccel.value() = 0.0;
       }

       // Direction vector for acceleration due to gravity
       if(forceFoamDict.found("gVector"))
       {
          gVector = vector(forceFoamDict.lookup("gVector"));
          gVector /= mag(gVector);
       }

       if(forceFoamDict.found("CgCenter"))
       {
          CgCenter = vector(forceFoamDict.lookup("CgCenter"));

       }

    }

/* ------------------ End of readForceBalanceControls.H --------------------------------*/
